@@ -1,12 +1,16 @@
 import warnings
-from datetime import date
-from typing import Any, Optional, Sequence, TYPE_CHECKING, Union
+from datetime import date, datetime
+from typing import TYPE_CHECKING, Any, Optional, Sequence, Union
+
 
 from calcbench.api_query_params import (
-    CompanyIdentifierScheme,
+    APIQueryParams,
+    CompaniesParameters,
     CompanyIdentifiers,
+    CompanyIdentifierScheme,
     Period,
     PeriodArgument,
+    PeriodParameters,
     PeriodType,
 )
 
@@ -19,11 +23,7 @@ else:
     except ImportError:
         from typing_extensions import TypedDict
 
-
-from calcbench.api_client import (
-    _json_POST,
-    logger,
-)
+from calcbench.api_client import _json_POST, logger
 
 try:
     import numpy as np
@@ -32,7 +32,6 @@ try:
     period_number = pd.api.types.CategoricalDtype(  # type: ignore
         categories=[1, 2, 3, 4, 5, 6, 0], ordered=True
     )  # So annual is last in sorting.  5 and 6 are first half and 3QCUM.
-
 except ImportError:
     "Can't find pandas, won't be able to use the functions that return DataFrames."
     pass
@@ -72,25 +71,27 @@ class StandardizedPoint(TypedDict):
 def standardized_raw(
     company_identifiers: CompanyIdentifiers = [],
     metrics: Sequence[str] = [],
-    start_year: int = None,
+    start_year: Optional[int] = None,
     start_period: PeriodArgument = None,
-    end_year: int = None,
+    end_year: Optional[int] = None,
     end_period: PeriodArgument = None,
     entire_universe=False,
-    accession_id: int = None,
+    accession_id: Optional[int] = None,
     point_in_time: bool = False,
     include_trace: bool = False,
-    update_date: date = None,
+    update_date: Optional[date] = None,
     all_history=False,
-    year: int = None,
+    year: Optional[int] = None,
     period: PeriodArgument = None,
-    period_type: PeriodType = None,
+    period_type: Optional[PeriodType] = None,
     include_preliminary: bool = False,
     use_fiscal_period: bool = False,
     all_face: bool = False,
     all_footnotes: bool = False,
     include_xbrl: bool = False,
-    exclude_errors: bool = True,
+    filing_id: Optional[int] = None,
+    all_non_GAAP: bool = False,
+    all_metrics=False,
 ) -> Sequence[StandardizedPoint]:
     """Standardized data.
 
@@ -109,6 +110,9 @@ def standardized_raw(
     :param period_type: Either "annual" or "quarterly"
     :param include_preliminary: Include data from non-XBRL 8-Ks and press releases.
     :param exclude_errors: Run another level of error detections, only works for PIT preliminary
+    :param filing_id: Filing id for which to get data.  corresponds to the filing_id in the objects returned by the filings API.
+    :param all_non_GAAP: include all non-GAAP metrics from earnings press releases such as EBITDA_NonGAAP.  This is implied when querying by `filing_id`.
+    :param all_metrics: All metrics.
     :return: A list of dictionaries with keys ['ticker', 'calendar_year', 'calendar_period', 'metric', 'value'].
 
     """
@@ -116,9 +120,10 @@ def standardized_raw(
         bool(company_identifiers),
         bool(entire_universe),
         bool(accession_id),
+        bool(filing_id),
     ].count(True) != 1:
         raise ValueError(
-            "Must pass either company_identifiers and accession id or entire_universe=True"
+            "Must pass either company_identifiers, accession id, filing_id, or entire_universe=True"
         )
     if not any(
         [
@@ -127,13 +132,14 @@ def standardized_raw(
             update_date,
             year,
             accession_id,
+            filing_id,
             all_history,
             period_type,
         ]
     ):
         raise ValueError("Need to specify a period qualifier")
 
-    if accession_id and any(
+    if (accession_id or filing_id) and any(
         [
             company_identifiers,
             start_year,
@@ -146,7 +152,7 @@ def standardized_raw(
         ]
     ):
         raise ValueError(
-            "Accession IDs are specific to a filing, no other qualifiers make sense."
+            "Accession/Filing IDs are specific to a filing, no other qualifiers make sense."
         )
 
     if period is not None:
@@ -183,6 +189,13 @@ def standardized_raw(
             "specifying metrics with all_face or all_footnotes does not make sense"
         )
 
+    if not any(
+        [all_face, all_footnotes, metrics, filing_id, all_non_GAAP, all_metrics]
+    ):
+        raise ValueError(
+            "need to specify a metrics argument, 'all_face', 'all_foonotes', 'all_non_GAAP', 'all_metrics' or a metric"
+        )
+
     if isinstance(metrics, str):
         raise TypeError("metrics should be a list of strings")
 
@@ -213,7 +226,26 @@ def standardized_raw(
         end_period = int(end_period)  # type: ignore
     except (ValueError, TypeError):
         pass
-    payload = {
+
+    period_parameters: PeriodParameters = {
+        "year": start_year,
+        "period": start_period,
+        "endYear": end_year,
+        "endPeriod": end_period,
+        "allHistory": all_history,
+        "updateDate": update_date and update_date.isoformat(),
+        "periodType": period_type,
+        "useFiscalPeriod": use_fiscal_period,
+        "accessionID": accession_id,
+        "filingID": filing_id,
+    }  # type: ignore
+
+    companies_parameters: CompaniesParameters = {
+        "entireUniverse": entire_universe,
+        "companyIdentifiers": list(company_identifiers),
+    }
+
+    payload: APIQueryParams = {
         "pageParameters": {
             "metrics": metrics,
             "includeTrace": include_trace,
@@ -222,60 +254,96 @@ def standardized_raw(
             "allFootnotes": all_footnotes,
             "allface": all_face,
             "includeXBRL": include_xbrl,
-            "excludeErrors": exclude_errors,
-        },
-        "periodParameters": {
-            "year": start_year,
-            "period": start_period,
-            "endYear": end_year,
-            "endPeriod": end_period,
-            "allHistory": all_history,
-            "updateDate": update_date and update_date.isoformat(),
-            "periodType": period_type,
-            "useFiscalPeriod": use_fiscal_period,
-            "accessionID": accession_id,
-        },
-        "companiesParameters": {
-            "entireUniverse": entire_universe,
-            "companyIdentifiers": list(company_identifiers),
+            "allNonGAAP": all_non_GAAP,
+            "allMetrics": all_metrics,
         },
+        "periodParameters": period_parameters,
+        "companiesParameters": companies_parameters,
     }
 
     return _json_POST("mappedData", payload)
 
 
+ORDERED_COLUMNS = [
+    "ticker",
+    "metric",
+    "fiscal_year",
+    "fiscal_period",
+    "value",
+    "revision_number",
+    "preliminary",
+    "XBRL",
+    "date_reported",
+    "filing_type",
+    "CIK",
+    "calcbench_entity_id",
+    "period_start",
+    "period_end",
+    "calendar_year",
+    "calendar_period",
+    "filing_accession_number",
+    "trace_url",
+]
+
+
+def _build_data_frame(raw_data: Sequence[StandardizedPoint]) -> "pd.DataFrame":
+    """
+    The order of the columns should remain constant
+    """
+    if not raw_data:
+        return pd.DataFrame()
+    data = pd.DataFrame(raw_data)
+    new_columns = list(set(data.columns) - set(ORDERED_COLUMNS))
+    data = data.reindex(columns=ORDERED_COLUMNS + new_columns)
+    data = data.drop(columns=["trace_facts"], errors="ignore")  # type: ignore
+    return data
+
+
 def point_in_time(
     company_identifiers: CompanyIdentifiers = [],
     all_footnotes: bool = False,
     metrics: Sequence[str] = [],
     all_history: bool = False,
     entire_universe: bool = False,
-    start_year: int = None,
+    start_year: Optional[int] = None,
     start_period: PeriodArgument = None,
-    end_year: int = None,
+    end_year: Optional[int] = None,
     end_period: PeriodArgument = None,
-    period_type: PeriodType = None,
+    period_type: Optional[PeriodType] = None,
     use_fiscal_period: bool = False,
     include_preliminary: bool = False,
     all_face: bool = False,
     include_xbrl: bool = True,
-    accession_id: int = None,
+    accession_id: Optional[int] = None,
     include_trace: bool = False,
     set_index: bool = False,
-    exclude_errors: bool = False,
+    _point_in_time_mode=True,
+    filing_id: Optional[int] = None,
+    all_non_GAAP: bool = False,
 ) -> "pd.DataFrame":
     """Point-in-Time Data
 
-    Standardized data with a timestamp when it was published by Calcbench
+    Standardized data with a timestamp when it was published by Calcbench.
 
-    :param accession_id: Unique identifier for the filing for which to recieve data.  Pass this to recieve data for one filing.  Same as filing_id in filings objects
+
+    A record is returned for each filing in which a metric value changed (was revised).
+
+    If the company files an 8-K with revenue = $100 then a week later files a 10-K with revenue = $100 one record will be returned for that period.  It will have preliminary=True and XBRL=True.
+
+    If the company files an 8-K with revenue = $100 and then a week later files a 10-K with revenue = $200, two records will be returned.  One with revenue = $100, a revision_number=0, date_reported of 8-K, preliminary=True and XBRL=False.  You will see a second line with revenue = $200, a revision_number=1, and date_reported of the 10-K, preliminary=False and XBRL=False.
+
+    If the value is revised in subsequent XBRL filings you will see a record for each filing with an incremented revision number.
+
+    :param accession_id: Unique identifier for the filing for which to recieve data.  Pass this to recieve data for one filing.  Same as calcbench_id in filings objects
     :param all_face: Retrieve all of the points from the face financials, income/balance/statement of cash flows
     :param all_footnotes: Retrive all of the points from the footnotes to the financials
     :param include_preliminary: Include facts from non-XBRL earnings press-releases and 8-Ks.
     :param include_xbrl: Include facts from XBRL 10-K/Qs.
     :param include_trace: Include a URL that points to the source document.
     :param set_index: Set a useful index on the returned DataFrame
-    :param exclude_errors:  Run the error detection
+    :param _point_in_time_mode: DO NOT USE.  For debugging only.
+    :param filing_id: Filing id for which to get data.  corresponds to the filing_id in the objects returned by the filings API.
+    :param all_non_GAAP: include all non-GAAP metrics from earnings press releases such as EBITDA_NonGAAP.  This is implied when querying by `filing_id`.
     :return: DataFrame of facts
 
     Columns:
@@ -285,12 +353,18 @@ def point_in_time(
        The value of the fact
     revision_number
        0 indicates an original, unrevised value for this fact. 1, 2, 3... indicates subsequent revisions to the fact value.  https://knowledge.calcbench.com/hc/en-us/search?utf8=%E2%9C%93&query=revisions&commit=Search
+    preliminary
+        True indicates the number was parsed from non-XBRL 8-K or press release from the wire
+    XBRL
+        Indicates the number was parsed from XBRL
     period_start
        First day of the fiscal period for this fact
     period_end
        Last day of the fiscal period for this fact
     date_reported
-       Timestamp when Calcbench published this fact
+       Timestamp (EST) when Calcbench published this fact.
+
+       In some cases, particularly prior to 2015, this will be the filing date of the document as recorded by the SEC.  To exclude these points remove points where the hour is 0.
     metric
        The metric name, see the definitions @ https://www.calcbench.com/home/standardizedmetrics
     calendar_year
@@ -327,7 +401,7 @@ def point_in_time(
         company_identifiers=company_identifiers,
         all_face=all_face,
         all_footnotes=all_footnotes,
-        point_in_time=True,
+        point_in_time=_point_in_time_mode,
         metrics=metrics,
         all_history=all_history,
         entire_universe=entire_universe,
@@ -341,39 +415,14 @@ def point_in_time(
         accession_id=accession_id,
         include_trace=include_trace,
         include_xbrl=include_xbrl,
-        exclude_errors=exclude_errors,
+        filing_id=filing_id,
+        all_non_GAAP=all_non_GAAP,
     )
 
     if not data:
         return pd.DataFrame()
+    data = _build_data_frame(data)
 
-    data = pd.DataFrame(data)
-
-    # specify the order but include all columns
-    ordered_columns = [
-        "ticker",
-        "metric",
-        "fiscal_year",
-        "fiscal_period",
-        "revision_number",
-        "value",
-        "preliminary",
-        "XBRL",
-        "date_reported",
-        "filing_type",
-        "CIK",
-        "calcbench_entity_id",
-        "period_start",
-        "period_end",
-        "calendar_year",
-        "calendar_period",
-        "filing_accession_number",
-        "trace_url",
-    ]
-
-    new_columns = list(set(data.columns) - set(ordered_columns))
-    data = data.reindex(columns=ordered_columns + new_columns)
-    data = data.drop(columns=["trace_facts"], errors="ignore")  # type: ignore
     sort_columns = ["ticker", "metric"]
 
     if "calendar_period" in data.columns:
@@ -403,7 +452,9 @@ def point_in_time(
             elif period_type == PeriodType.Annual:
                 data["period"] = pd.PeriodIndex(data["fiscal_year"], freq="A")
         data.drop(["fiscal_year", "fiscal_period"], axis=1, inplace=True)
-        data = data.set_index(["ticker", "metric", "period", "revision_number"])
+        data = data.set_index(
+            ["ticker", "metric", "period", "date_reported"]
+        ).sort_index()
     else:
         data = data.sort_values(sort_columns).reset_index(drop=True)
     return data
@@ -412,20 +463,20 @@ def point_in_time(
 def standardized_data(
     company_identifiers: CompanyIdentifiers = [],
     metrics: Sequence[str] = [],
-    start_year: int = None,
+    start_year: Optional[int] = None,
     start_period: PeriodArgument = None,
-    end_year: int = None,
+    end_year: Optional[int] = None,
     end_period: PeriodArgument = None,
     entire_universe: bool = False,
     point_in_time: bool = False,
-    year: int = None,
+    year: Optional[int] = None,
     period: PeriodArgument = None,
     all_history: bool = False,
-    period_type: PeriodType = None,
+    period_type: Optional[PeriodType] = None,
     trace_hyperlinks: bool = False,
     use_fiscal_period: bool = False,
     company_identifier_scheme: CompanyIdentifierScheme = CompanyIdentifierScheme.Ticker,
-    accession_id: int = None,
+    accession_id: Optional[int] = None,
 ) -> "pd.DataFrame":
     """Standardized Data.
 
@@ -544,6 +595,82 @@ def standardized_data(
     return data
 
 
+def standardized(
+    company_identifiers: CompanyIdentifiers = [],
+    metrics: Sequence[str] = [],
+    fiscal_year: Optional[int] = None,
+    fiscal_period: PeriodArgument = None,
+    point_in_time: bool = False,
+    filing_id: Optional[int] = None,
+):
+    """Standardized Numeric Data.
+
+
+
+    The data behind the multi-company page, https://www.calcbench.com/multi.
+
+    Example https://github.com/calcbench/notebooks/blob/master/python_client_api_demo.ipynb
+
+    :param company_identifiers: Tickers/CIK codes. eg. ['msft', 'goog', 'appl', '0000066740'].  If not specified get data for all companies.
+    :param metrics: Standardized metrics.  Full list @ https://www.calcbench.com/home/standardizedmetrics eg. ['revenue', 'accountsreceivable'].  If not specified get all metrics.
+    :param fiscal_year: Fiscal year for which to get data.  If not specified get all history.
+    :param fiscal_period: Fiscal period for which to get data.  If not specified get all history.
+    :param point_in_time: Include timestamps when data was published and revision chains.
+    :param filing_id: Filing ID for which to get data.  Get all of the data reported in this filing.
+    :return: Dataframe
+
+    Usage::
+
+      >>> d = calcbench.standardized(company_identifiers=['msft'],
+      >>>                                 point_in_time=True,)
+      >>> )
+      >>> # Put the data in a format amiable to arithmetic on columns
+      >>> d = calcbench.standardized(company_identifiers=['msft', 'orcl'], metrics=['StockholdersEquity', 'NetIncome'])
+      >>> d = d.unstack("metric")["value"]
+      >>> return_on_equity = d['NetIncome'] / d['StockholdersEquity']
+    """
+    data = standardized_raw(
+        company_identifiers=company_identifiers,
+        all_history=not fiscal_year,
+        year=fiscal_year,
+        period=fiscal_period,
+        point_in_time=point_in_time,
+        metrics=metrics,
+        entire_universe=not (company_identifiers or filing_id),
+        filing_id=filing_id,
+        all_metrics=not metrics,
+        use_fiscal_period=True,
+    )
+
+    data = _build_data_frame(data)
+    if data.empty:
+        return data
+    data["fiscal_period"] = (
+        data["fiscal_year"].astype(str) + "-" + data["fiscal_period"].astype(str)
+    )
+    data = data.drop("fiscal_year", axis=1)
+
+    if "calendar_period" in data.columns:
+        data.calendar_period = data.calendar_period.astype(period_number)  # type: ignore
+
+    if "calendar_period" in data.columns:
+        data.calendar_period = data.calendar_period.astype(period_number)  # type: ignore
+
+    for date_column in ["date_reported", "period_end", "period_start"]:
+        if date_column in data.columns:
+            data[date_column] = pd.to_datetime(data[date_column], errors="coerce")  # type: ignore
+    index_columns = [
+        "ticker",
+        "metric",
+        "fiscal_period",
+    ]
+    if point_in_time:
+        index_columns = index_columns + ["date_reported"]
+        data["date_downloaded"] = datetime.now()
+    data = data.set_index(index_columns)
+    return data
+
+
 def _build_quarter_period(
     data_point: StandardizedPoint, use_fiscal_period: bool
 ) -> "pd.Period":
@@ -574,4 +701,4 @@ def _build_annual_period(
 
 normalized_data = standardized_data  # used to call it normalized_data.
 normalized_dataframe = standardized_data
-normalized_raw = standardized_raw
\ No newline at end of file
+normalized_raw = standardized_raw
